/*************************************************************************************
Data Visualization - field worker data demo for Project3              
Tested in Processing 4                                   
                                                     
Author: Yixuan Li                      
Supervisor: George Legrady                           

Purpose: Show the pipeline of conducting Project 3
*************************************************************************************/

import peasy.*;
import peasy.org.apache.commons.math.*;
import peasy.org.apache.commons.math.geometry.*;
import controlP5.*;

ControlGroup l;
ControlP5 controlP5;

boolean version1 = true;
boolean version2 = false;
boolean version3 = false;
boolean hoverEffect = false;
boolean logEffect = true;
boolean hEffect = true;
boolean tEffect = false;
boolean manual = true;

// Load the data
String hFile = "data_Hum.csv";
String hString[];
String tFile = "data_Temp.csv";
String tString[];

// Compute the total date of the dataset
int totalDate = int(dateParser("10/2/21")-dateParser("7/17/21")+1);
float logBase = 1.5;
String dates[] = new String[totalDate];
float cVolume[][][] = new float[50][totalDate][24];
float hVolume[][][] = new float[50][totalDate][24];
float tVolume[][][] = new float[50][totalDate][24];


float minHValue = 10000;
float maxHValue = 0;
float minTValue = 10000;
float maxTValue = 0;
float zBase = 50; // height distance between gray platform and the two infographs

PeasyCam cam; // activate the PeasyCam interaction


float adjustX = 20; //spacing of x-axis
float adjustY = 20; //spacing of y-axis
float adjustZ = 35; //spacing of z-axis
int spaceZ = 60; //distance between each layer

int X = (int)adjustX* totalDate; //x-axis lenght
int Y = (int)adjustY* 24; //y-axis lenght
int Z = (int)spaceZ*50; //peasycam Z depth

PFont axisText;
PFont hoverText;

Integrator slide;
boolean lockGUI;
//----------------------------------------------------------
void setup() {
  colorMode(HSB, 255); //set color mode to HSB
  
  //fullScreen(P3D);
  size(1400, 800, P3D);
  smooth(); //Anti-aliasing of texts and visuals for a smooth visual effect

  // PeasyCam (this, x position, y position, z position, viewing distance (larger number means closer viewing)
  cam = new PeasyCam(this, 1800);
  // set onscreen visible distances for hte PeasyCam, otherwise it may not fully show
  cam.setMinimumDistance(1);//1
  cam.setMaximumDistance(Z);//was 5000, now 2000
  
  // Set the angle of view - like changing the lens on a camera PI/2 (wide angle); PI/6 normal lens; PI/12 (telephoto)
  float fov = PI/8; // field of view try between PI/2 to PI/10
  float cameraZ = (height/2.0) / tan(PI/6);
  //perspective(field of view in radians; aspect/ratio of width to height; zNear - Z position of nearest clipping, zFar - Z position of farthest clipping)
  //default values are : perspective(PI/3.0, width/height, cameraZ/10.0, cameraZ*10.0) where cameraZ is ((height/2.0) / tan(PI*60.0/360.0));
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*10.0); // if using 0.001 instead of cameraZ/10.0 so that all things close to the screen can be seen - get noise
  initializeData();
  //Set the font
  axisText = createFont("Arial", 14);
  hoverText = createFont("Arial", 120);
  
  slide = new Integrator(75,0.3,0.8);
  setupGUI();
 
}
//----------------------------------------------------------
void draw() {
  background(0);
  
  
  //axis length and Z depth
  X = (int)adjustX* totalDate; //x-axis lenght
  Y = (int)adjustY* 24; //y-axis lenght
  Z = (int)(spaceZ)*50; //peasycam Z depth
  pushMatrix();
  rotateX(PI/2);
  rotateZ(PI);
  translate(-X/2, -Y/2,-Z/2);

  drawPlatform(); //draw the axis

  
  // If select the humidity data
  if(hEffect){
    cVolume = hVolume;
  // draw visualization model 1
  if(version1==true){
    drawVersion1(false, hEffect);}
  // draw visualization model 2
  if(version2==true){
    drawVersion2(false, hEffect);}
  // draw visualization model 3
  if(version3==true){
    drawVersion3(false, hEffect);}} 
  // If select the temperature data
  if(tEffect){
    cVolume = tVolume;
  // draw visualization model 1
  if(version1==true){
    drawVersion1(tEffect, false);}
  // draw visualization model 2
  if(version2==true){
    drawVersion2(tEffect, false);}
  // draw visualization model 3
  if(version3==true){
    drawVersion3(tEffect, false);} }
  popMatrix();
  title();
  slide.update();
  // When the mouse is close to the bottom, open the manual
  if((mouseY>height-150)){
    cam.setActive(false);
    lockGUI = true;
    slide.target(0);
    }
    else if(mouseY<height-150){
      slide.target(75);
      cam.setActive(true);}
    if (lockGUI == false){
      slide.target(75);}
   
  if (l.getPosition()[1] != height-100+slide.value){
    println(l.getPosition()[1]);
    l.setPosition(l.getPosition()[0], height-100+slide.value);
    
}

drawGUI();
}
